@@ -8,37 +8,41 @@
 #include "rugged.h"
 
 const char *RUGGED_ERROR_NAMES[] = {
-	"None",            /* GITERR_NONE */
-	"NoMemError",      /* GITERR_NOMEMORY */
-	"OSError",         /* GITERR_OS */
-	"InvalidError",    /* GITERR_INVALID */
-	"ReferenceError",  /* GITERR_REFERENCE */
-	"ZlibError",       /* GITERR_ZLIB */
-	"RepositoryError", /* GITERR_REPOSITORY */
-	"ConfigError",     /* GITERR_CONFIG */
-	"RegexError",      /* GITERR_REGEX */
-	"OdbError",        /* GITERR_ODB */
-	"IndexError",      /* GITERR_INDEX */
-	"ObjectError",     /* GITERR_OBJECT */
-	"NetworkError",    /* GITERR_NET */
-	"TagError",        /* GITERR_TAG */
-	"TreeError",       /* GITERR_TREE */
-	"IndexerError",    /* GITERR_INDEXER */
-	"SslError",        /* GITERR_SSL */
-	"SubmoduleError",  /* GITERR_SUBMODULE */
-	"ThreadError",     /* GITERR_THREAD */
-	"StashError",      /* GITERR_STASH */
-	"CheckoutError",   /* GITERR_CHECKOUT */
-	"FetchheadError",  /* GITERR_FETCHHEAD */
-	"MergeError",      /* GITERR_MERGE */
-	"SshError",        /* GITERR_SSH */
-	"FilterError",     /* GITERR_FILTER */
-	"RevertError",     /* GITERR_REVERT */
-	"CallbackError",   /* GITERR_CALLBACK */
-	"CherrypickError", /* GITERR_CHERRYPICK */
-	"DescribeError",   /* GITERR_DESCRIBE */
-	"RebaseError",     /* GITERR_REBASE */
-	"FilesystemError", /* GITERR_FILESYSTEM */
+	"None",            /* GIT_ERROR_NONE */
+	"NoMemError",      /* GIT_ERROR_NOMEMORY */
+	"OSError",         /* GIT_ERROR_OS */
+	"InvalidError",    /* GIT_ERROR_INVALID */
+	"ReferenceError",  /* GIT_ERROR_REFERENCE */
+	"ZlibError",       /* GIT_ERROR_ZLIB */
+	"RepositoryError", /* GIT_ERROR_REPOSITORY */
+	"ConfigError",     /* GIT_ERROR_CONFIG */
+	"RegexError",      /* GIT_ERROR_REGEX */
+	"OdbError",        /* GIT_ERROR_ODB */
+	"IndexError",      /* GIT_ERROR_INDEX */
+	"ObjectError",     /* GIT_ERROR_OBJECT */
+	"NetworkError",    /* GIT_ERROR_NET */
+	"TagError",        /* GIT_ERROR_TAG */
+	"TreeError",       /* GIT_ERROR_TREE */
+	"IndexerError",    /* GIT_ERROR_INDEXER */
+	"SslError",        /* GIT_ERROR_SSL */
+	"SubmoduleError",  /* GIT_ERROR_SUBMODULE */
+	"ThreadError",     /* GIT_ERROR_THREAD */
+	"StashError",      /* GIT_ERROR_STASH */
+	"CheckoutError",   /* GIT_ERROR_CHECKOUT */
+	"FetchheadError",  /* GIT_ERROR_FETCHHEAD */
+	"MergeError",      /* GIT_ERROR_MERGE */
+	"SshError",        /* GIT_ERROR_SSH */
+	"FilterError",     /* GIT_ERROR_FILTER */
+	"RevertError",     /* GIT_ERROR_REVERT */
+	"CallbackError",   /* GIT_ERROR_CALLBACK */
+	"CherrypickError", /* GIT_ERROR_CHERRYPICK */
+	"DescribeError",   /* GIT_ERROR_DESCRIBE */
+	"RebaseError",     /* GIT_ERROR_REBASE */
+	"FilesystemError", /* GIT_ERROR_FILESYSTEM */
+	"PathError",       /* GIT_ERROR_PATCH */
+	"WorktreeError",   /* GIT_ERROR_WORKTREE */
+	"SHA1Error",       /* GIT_ERROR_SHA1 */
+	"HTTPError",       /* GIT_ERROR_HTTP */
 };
 
 #define RUGGED_ERROR_COUNT (int)((sizeof(RUGGED_ERROR_NAMES)/sizeof(RUGGED_ERROR_NAMES[0])))
